@@ -20,7 +20,7 @@ en = {
     "COLAB_DEBUG_TRUE": "Colab notebook detected. This cell will run indefinitely so that you can see errors and logs. "
                       "To turn off, set debug=False in launch().",
     "COLAB_DEBUG_FALSE": "Colab notebook detected. To show errors in colab notebook, set debug=True in launch()",
-    "SHARE_LINK_MESSAGE": "This share link will expire in 24 hours. If you need a permanent link, visit: https://gradio.app/hosted (NEW!)",
+    "SHARE_LINK_MESSAGE": "This share link will expire in 24 hours. If you need a permanent link, visit: https://gradio.app/introducing-hosted (NEW!)",
     "SHARE_LINK_DISPLAY": "Running on External URL: {}",
     "INLINE_DISPLAY_BELOW": "Interface loading below...",
     "TIP_INTERPRETATION": "Tip: Add interpretation to your model by simply adding `interpretation=\"default\"` to `Interface()`",
@@ -28,7 +28,7 @@ en = {
 }
 
 try:
-    updated_messaging = requests.get(MESSAGING_API_ENDPOINT).json()
+    updated_messaging = requests.get(MESSAGING_API_ENDPOINT, timeout=3).json()
     en.update(updated_messaging)
-except (requests.ConnectionError, json.decoder.JSONDecodeError):  # Use default messaging
+except (requests.ConnectionError, requests.exceptions.ReadTimeout, json.decoder.JSONDecodeError):  # Use default messaging
     pass
